from argon2 import PasswordHasher
from argon2.exceptions import VerificationError
from cloud_inquisitor.constants import MSG_INVALID_USER_OR_PASSWORD, ROLE_ADMIN, ROLE_USER
from cloud_inquisitor.plugins import BaseAuthPlugin, BaseView
from cloud_inquisitor.schema import Role, User
from cloud_inquisitor.utils import generate_csrf_token, generate_jwt_token, generate_password, hash_password
from flask import session, request

from cloud_inquisitor.database import db

ph = PasswordHasher()


class BaseLocalAuthView(BaseView):
    name = 'Local Authentication'
    ns = 'auth_local'


class LocalAuthLogin(BaseLocalAuthView):
    URLS = ['/auth/local/login']

    def post(self):
        self.reqparse.add_argument('username', type=str, required=True)
        self.reqparse.add_argument('password', type=str, required=True)
        args = self.reqparse.parse_args()

        try:
            user = db.User.filter(
                User.username == args['username'],
                User.auth_system == self.name
            ).first()

            if not user:
                self.log.warn('Authentication attemp for unknown user {} from {}'.format(
                    args['username'],
                    request.remote_addr
                ))
                return self.make_response({
                    'message': MSG_INVALID_USER_OR_PASSWORD
                }, 400)

            ph.verify(user.password, args['password'])

            # Setup the user session with all the information required
            session['user'] = user
            session['csrf_token'] = generate_csrf_token()
            session['accounts'] = [x.account_id for x in db.Account.all() if x.user_has_access(user)]

            token = generate_jwt_token(user, self.name)

            return self.make_response({
                'authToken': token,
                'csrfToken': session['csrf_token']
            }, 200)

        except VerificationError:
            self.log.warn('Failed verifying password for {} from {}'.format(
                args['username'],
                request.remote_addr
            ))
            return self.make_response({
                'message': MSG_INVALID_USER_OR_PASSWORD
            }, 400)

        except Exception as ex:
            self.log.exception('Unknown error occured while attempting to login user {} from {}: {}'.format(
                args['username'],
                request.remote_addr,
                ex
            ))

        finally:
            db.session.rollback()


class LocalAuthLogout(BaseLocalAuthView):
    URLS = ['/auth/local/logout']

    def get(self):
        pass


class LocalAuth(BaseAuthPlugin):
    name = 'Local Authentication'
    ns = 'auth_local'
    views = (LocalAuthLogin, LocalAuthLogout)
    readonly = False
    login = {'state': 'auth.login'}
    logout = '/auth/local/logout'

    def bootstrap(self):
        admin_user = db.User.find_one(
            User.username == 'admin',
            User.auth_system == self.name
        )

        if not admin_user:
            roles = db.Role.filter(Role.name.in_((ROLE_ADMIN, ROLE_USER))).all()
            admin_password = generate_password()
            admin_user = User()

            admin_user.username = 'admin'
            admin_user.auth_system = self.name
            admin_user.password = hash_password(admin_password)
            db.session.add(admin_user)
            db.session.commit()
            db.session.refresh(admin_user)
            User.add_role(admin_user, roles)

            self.log.error('Created admin account for local authentication, username: admin, password: {}'.format(
                admin_password
            ))

        else:
            self.log.debug('Local Auth admin user already exists, skipping')



import os
from codecs import open

import setuptools


path = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(path, 'README.rst')) as fd:
    long_desc = fd.read()

setuptools.setup(
    name='cinq-auth-local',
    use_scm_version=True,

    entry_points={
        'cloud_inquisitor.plugins.auth': [
            'auth_local = cinq_auth_local:LocalAuth'
        ]
    },

    packages=setuptools.find_packages(),
    setup_requires=['setuptools_scm'],
    install_requires=[
        'cloud_inquisitor~=2.0',
        'flask>=0.12.2',
        'argon2-cffi>=16.3.0',
    ],
    extras_require={
        'dev': [],
        'test': [],
    },

    # Metadata for the project
    description='Local DB based user authentication',
    long_description=long_desc,
    url='https://github.com/RiotGames/cinq-auth-local/',
    author='Riot Games Security',
    author_email='security@riotgames.com',
    license='License :: OSI Approved :: Apache Software License',
    classifiers=[
        # Current project status
        'Development Status :: 4 - Beta',

        # Audience
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',

        # License information
        'License :: OSI Approved :: Apache Software License',

        # Supported python versions
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',

        # Frameworks used
        'Framework :: Flask',
        'Framework :: Sphinx',

        # Supported OS's
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Unix',

        # Extra metadata
        'Environment :: Console',
        'Natural Language :: English',
        'Topic :: Security',
        'Topic :: Utilities',
    ],
    keywords='cloud security',
)
